package cz.cvut.fel.tik.adaptivehuffman;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import cz.cvut.fel.tik.adaptivehuffman.decoder.*;
import cz.cvut.fel.tik.adaptivehuffman.encoder.*;
import cz.cvut.fel.tik.adaptivehuffman.factory.CodingFactory;
import cz.cvut.fel.tik.adaptivehuffman.factory.FGKFactory;
import cz.cvut.fel.tik.adaptivehuffman.factory.VitterFactory;

public class AdaptiveHuffmanMain {

	public static final String DEFAULT_MESSAGE = "this is a test message"; 
	
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		
		String message;
		
		CodingFactory factory;
		
		
		if (args.length != 2) {
			message = DEFAULT_MESSAGE;
			factory = new VitterFactory();
		} else {
			message = args[1];
			factory = "fgk".equals(args[0]) ? new FGKFactory() : new VitterFactory();
		}
		/*
		//file-reader. Tested almost 100kB of lorem ipsum
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		String s;
		String msg = "";
		while((s = reader.readLine()) != null) 	msg += s;
		message = msg;
		*/
		
		long start = System.currentTimeMillis();
		
		Encoder encoder = factory.createEncoder();
		String encodedMessage = encoder.encode(message);
		
		long mid = System.currentTimeMillis();
		
		Decoder decoder = factory.createDecoder();
		String decodedMessage = decoder.decode(encodedMessage);
		
		long end = System.currentTimeMillis();
		
		System.out.println("algorithm: " + factory.toString());
		System.out.println("original message: " + message);
		System.out.println("encoded message: " + encodedMessage);
		System.out.println("decoded message: " + decodedMessage);
		System.out.println("result info: " + (message.equals(decodedMessage)? "sent and received messages are equal" : "666 ALERT ERROR ATTENTION"));
		System.out.println("encoding time [ms]: " + (mid - start) + "  decoding time [ms]: " + (end - mid) + "  total time [ms]: " + (end - start));
		System.out.println("original size: " + (message.length() * 8) + "b" + "  encoded size: " + (encodedMessage.length()) + "b" + "  compression ratio: " + ((double)encodedMessage.length() / (message.length() * 8)));
		
	}

}
