package cz.cvut.fel.tik.adaptivehuffman.tree;

import java.util.HashMap;
import java.util.Map;

import cz.cvut.fel.tik.adaptivehuffman.util.Utility;

public class FGKTree implements Tree {
	
	private int lastId = 0;
	
	private Node root;
	
	private Map<Character, Node> symbolHash = new HashMap<Character, Node>();
	
	public FGKTree() {
		root = new Node(NYT, 0, ++lastId, 0, "");
		symbolHash.put(NYT, root);
	}
	
	public String getBitValueOf(char c) {
		if(symbolHash.containsKey(c)) {
			Node n = symbolHash.get(c);
			String ret = n.getBit;
			updateTree(n);
//			System.out.println("Encoded "+c+" as "+ret+", its node has value "+n.value);
			return ret;
		} else {
			Node nyt = symbolHash.get(NYT);
			symbolHash.remove(NYT);
			createLeftSuccessor(nyt, NYT);
			createRightSuccessor(nyt, c);
			String ret = nyt.getBit + Utility.charToBit(c);
//			System.out.println("Encoded "+c+" as "+nyt.getBit+" "+charToBit(c)+", it was a new node.");
			updateTree(nyt.right);
			return(ret);
		}
	}
	
	public char decodeLeaf(String message) {
		Node n = root;
		for(int i = 0; i < message.length(); i++) {
			char c = message.charAt(i);
			if(c == '0') n = n.left;
			else n = n.right;
		}
		if(n.isLeaf()) {
			char c = n.symbol;
			if(n.symbol != NYT) updateTree(n);
			return c;
		}
		return NULL;
	}
	
	private void updateTree(Node u) {
		while(u.parent != null) {
			Node leader;
			if(!u.isLeaf()) {
				leader = findLeader(root, u.value, false);
				if(leader.getBit != null){
					if(leader != u){
						Node ln = u.left;
						Node rn = u.right;
						Node p = u.parent;
						
						u.left = leader.left;
						u.right = leader.right;
						u.parent = leader.parent;
						leader.left = ln;
						leader.right = rn;
						leader.parent = p;
					}
				}
				
			}
			u.value++;
			u = u.parent;
			
		}
		u.value++;
	}

	private Node findLeader(Node rn, int value, boolean lookingForLeaf) {
		if(rn.isLeaf() == lookingForLeaf) {
			if(rn.value == value) return rn;
		}
		
		Node l = rn.left;
		Node r = rn.right;
		Node ls = new Node(NYT);
		Node rs = new Node(NYT);
		ls.level = Integer.MAX_VALUE;
		rs.level = Integer.MAX_VALUE;
		if(r != null) {
		if(r.isLeaf() == lookingForLeaf) {
			 if(r.value == value) return r;
		} else rs = findLeader(r, value, lookingForLeaf);
		}
		if(l != null) {
		if(l.isLeaf() == lookingForLeaf) {
			if(l.value == value) return l;
		} else ls = findLeader(l, value, lookingForLeaf);
		}
		if(rs.level <= ls.level) return rs;
		return ls;
		
	}

	private void createLeftSuccessor(Node n, char c) {
		Node newN = new Node(c, 0, ++lastId, n.level + 1, n.getBit + "0", n);
		n.left = newN;
		symbolHash.put(c, newN);
	}
	
	private void createRightSuccessor(Node n, char c) {
		Node newN = new Node(c, 0, ++lastId, n.level + 1, n.getBit + "1", n);
		n.right = newN;
		symbolHash.put(c, newN);
	}

	public char addNewChar(String substring) {
		//System.out.println(substring);
		char c = Utility.bitToChar(substring);
		Node nyt = symbolHash.get(NYT);
		symbolHash.remove(NYT);
		createLeftSuccessor(nyt, NYT);
		createRightSuccessor(nyt, c);
		updateTree(nyt.right);
		return c;
	}
	
	public void printSymbolHash() {
		System.out.println("---------------------------------------");
		for (Character c : symbolHash.keySet()) {
			Node n = symbolHash.get(c);
			System.out.println(c+" "+n.getBit+" - " +n.symbol);
		}
	}
	
}
